// Brushless motor control for Hoverboard Rover R15

// DIRection pins must be PWM capable
int M1_HALL_1 = A2;
int M1_HALL_2 = A1;
int M1_HALL_3 = A0;

int M1_P1_DIR =  3;
int M1_P1_EN =   2;

int M1_P2_DIR =  5;
int M1_P2_EN =   4;

int M1_P3_DIR =  6;
int M1_P3_EN =   7;

int M2_HALL_1 = A3;
int M2_HALL_2 = A4;
int M2_HALL_3 = A5;

int M2_P1_DIR =  9;
int M2_P1_EN =   8;

int M2_P2_DIR = 10;
int M2_P2_EN =  12;

int M2_P3_DIR = 11;
int M2_P3_EN =  13;



#define PWM1_MAX 15
#define PWM1_MIN 15
int PWM1 = 60;
int pwmCount = 0;
int pwm_delay_us = 1;

int i;
byte sensorState1 = 0;
byte sensorState2 = 0;
char comByte = 0;

int driveLength = 60;
int driveLength2 = 30;

int step1 = 1, step2 = 1;

byte setupMode = 0; //0 - Power, 1 - Speed

#define STEP_DELAY_MIN 500
#define STEP_DELAY_MAX 500
unsigned long stepDelay = 500;
unsigned long stepDelayDiff = 500;
unsigned long stepDelayMin = 200;// Limits maximum speed (lower = faster)



int pwmSwitch(int pwm_pin1, int pwm_pin2, int dutycycle, byte sensorComp1, byte sensorComp2){
  byte stepDone1 = 0, stepDone2 = 0;
  unsigned long runCount = 0;

  pwmCount = 0;
  runCount = 0;
  while(!(stepDone1 == 1 && stepDone2 == 1 && runCount > stepDelayMin)){
    sensorState1 = (digitalRead(M1_HALL_1)<<2) + (digitalRead(M1_HALL_2)<<1) + (digitalRead(M1_HALL_3));
    sensorState2 = (digitalRead(M2_HALL_1)<<2) + (digitalRead(M2_HALL_2)<<1) + (digitalRead(M2_HALL_3));
    if(pwmCount < dutycycle){
      digitalWrite(pwm_pin1, 1);
      digitalWrite(pwm_pin2, 1);
    }
    else{
      digitalWrite(pwm_pin1, 0);
      digitalWrite(pwm_pin2, 0);
    }
    delayMicroseconds(pwm_delay_us);
    if(pwmCount<255){
      pwmCount++;
    }
    else{
      pwmCount = 0;
    }
    if(sensorState1 == sensorComp1){
      stepDone1 = 1;
    }
    if(sensorState2 == sensorComp2){
      stepDone2 = 1;
    }
    runCount++;
    if(runCount>stepDelay){
      /*
      Serial.write("senor1=");
      Serial.print(sensorState1, BIN);
      Serial.print("(");
      Serial.print(sensorComp1, BIN);
      Serial.write("), senor2=");
      Serial.print(sensorState2, BIN);
      Serial.print("(");
      Serial.print(sensorComp2, BIN);
      Serial.write(")\n");
      */
      return -1;
    }
  }

  return 0;
}

int get_step1(){
  int step1 = 1;
  
  sensorState1 = (digitalRead(M1_HALL_1)<<2) + (digitalRead(M1_HALL_2)<<1) + (digitalRead(M1_HALL_3));

  if(sensorState1 == 0b00000110){
    step1 = 1;
  }
  if(sensorState1 == 0b00000100){
    step1 = 2;
  }
  if(sensorState1 == 0b00000101){
    step1 = 3;
  }
  if(sensorState1 == 0b00000001){
    step1 = 4;
  }
  if(sensorState1 == 0b00000011){
    step1 = 5;
  }
  if(sensorState1 == 0b00000010){
    step1 = 6;
  }

  return step1;
}

int get_step2(){
  int step2 = 2;
  
  sensorState2 = (digitalRead(M2_HALL_1)<<2) + (digitalRead(M2_HALL_2)<<1) + (digitalRead(M2_HALL_3));

  if(sensorState2 == 0b00000010){
    step2 = 1;
  }
  if(sensorState2 == 0b00000110){
    step2 = 2;
  }
  if(sensorState2 == 0b00000100){
    step2 = 3;
  }
  if(sensorState2 == 0b00000101){
    step2 = 4;
  }
  if(sensorState2 == 0b00000001){
    step2 = 5;
  }
  if(sensorState2 == 0b00000011){
    step2 = 6;
  }

  return step2;
}



void step_motors(int step1diff, int step2diff){
/*
  Serial.print("Step1=");
  Serial.print(step1, DEC);
  Serial.print(", Step2=");
  Serial.print(step2, DEC);
  Serial.print("\n");
*/

  byte hall_1 = 0;
  byte hall_2 = 0;
  byte hall_12 = 0;
  int pwm_pin_1 = 0;
  int pwm_pin_2 = 0;
  

  step1+=step1diff;
  if(step1 > 6){
    step1 = 1;
  }
  if(step1 < 1){
    step1 = 6;
  }

  if(step1 == 1){
    digitalWrite(M1_P2_DIR, 0);
    digitalWrite(M1_P1_EN, HIGH);
    digitalWrite(M1_P2_EN, HIGH);
    pwm_pin_1 = M1_P1_DIR;
    hall_1 = 0b00000110;
  }
  if(step1 == 2){
    digitalWrite(M1_P3_DIR, 0);
    digitalWrite(M1_P1_EN, HIGH);
    digitalWrite(M1_P3_EN, HIGH);
    pwm_pin_1 = M1_P1_DIR;
    hall_1 = 0b00000100;
  }
  if(step1 == 3){
    digitalWrite(M1_P3_DIR, 0);
    digitalWrite(M1_P2_EN, HIGH);
    digitalWrite(M1_P3_EN, HIGH);
    pwm_pin_1 = M1_P2_DIR;
    hall_1 = 0b00000101;
  }
  if(step1 == 4){
    digitalWrite(M1_P1_DIR, 0);
    digitalWrite(M1_P1_EN, HIGH);
    digitalWrite(M1_P2_EN, HIGH);
    pwm_pin_1 = M1_P2_DIR;
    hall_1 = 0b00000001;
  }
  if(step1 == 5){
    digitalWrite(M1_P1_DIR, 0);
    digitalWrite(M1_P1_EN, HIGH);
    digitalWrite(M1_P3_EN, HIGH);
    hall_1 = 0b00000011;
    pwm_pin_1 = M1_P3_DIR;
  }
  if(step1 == 6){
    digitalWrite(M1_P2_DIR, 0);
    digitalWrite(M1_P3_EN, HIGH);
    digitalWrite(M1_P2_EN, HIGH);
    hall_1 = 0b00000010;
    pwm_pin_1 = M1_P3_DIR;
  }


  step2+=step2diff;
  if(step2 > 6){
    step2 = 1;
  }
  if(step2 < 1){
    step2 = 6;
  }



  if(step2 == 1){
    digitalWrite(M2_P3_DIR, 0);
    digitalWrite(M2_P1_EN, HIGH);
    digitalWrite(M2_P3_EN, HIGH);
    pwm_pin_2 = M2_P1_DIR;
    hall_2 = 0b00000010;
  }
  if(step2 == 2){
    digitalWrite(M2_P2_DIR, 0);
    digitalWrite(M2_P1_EN, HIGH);
    digitalWrite(M2_P2_EN, HIGH);
    pwm_pin_2 = M2_P1_DIR;
    hall_2 = 0b00000110;
  }
  if(step2 == 3){
    digitalWrite(M2_P1_DIR, 0);
    digitalWrite(M2_P1_EN, HIGH);
    digitalWrite(M2_P2_EN, HIGH);
    pwm_pin_2 = M2_P2_DIR;
    hall_2 = 0b00000100;
  }
  if(step2 == 4){
    digitalWrite(M2_P3_DIR, 0);
    digitalWrite(M2_P2_EN, HIGH);
    digitalWrite(M2_P3_EN, HIGH);
    pwm_pin_2 = M2_P2_DIR;
    hall_2 = 0b00000101;
  }
  if(step2 == 5){
    digitalWrite(M2_P2_DIR, 0);
    digitalWrite(M2_P3_EN, HIGH);
    digitalWrite(M2_P2_EN, HIGH);
    hall_2 = 0b00000001;
    pwm_pin_2 = M2_P3_DIR;
  }
  if(step2 == 6){
    digitalWrite(M2_P1_DIR, 0);
    digitalWrite(M2_P1_EN, HIGH);
    digitalWrite(M2_P3_EN, HIGH);
    hall_2 = 0b00000011;
    pwm_pin_2 = M2_P3_DIR;
  }

  if(pwm_pin_1 == 0){
    pwm_pin_1 = pwm_pin_2;
  }
  if(pwm_pin_2 == 0){
    pwm_pin_2 = pwm_pin_1;
  }
  if(pwm_pin_1 !=0 && pwm_pin_2 != 0){
    pwmSwitch(pwm_pin_1, pwm_pin_2, PWM1, hall_1, hall_2);
  }
  digitalWrite(M1_P1_DIR, 0);
  digitalWrite(M1_P2_DIR, 0);
  digitalWrite(M1_P3_DIR, 0);
  digitalWrite(M1_P1_EN, LOW);
  digitalWrite(M1_P2_EN, LOW);
  digitalWrite(M1_P3_EN, LOW);

  digitalWrite(M2_P1_DIR, 0);
  digitalWrite(M2_P2_DIR, 0);
  digitalWrite(M2_P3_DIR, 0);
  digitalWrite(M2_P1_EN, LOW);
  digitalWrite(M2_P2_EN, LOW);
  digitalWrite(M2_P3_EN, LOW);

/*
  stepDelay -= stepDelayDiff;
  if(stepDelay < STEP_DELAY_MIN){
    stepDelay = STEP_DELAY_MIN;
  }
*/

}



// Initialize communication through USB
void establishContact() {
  while (Serial.available() <= 0) {
    Serial.print('X');   // send a capital X
    delay(100);
  }
}


// the setup routine runs once when you press reset:
void setup() {
  pinMode(M1_P1_DIR, OUTPUT);
  digitalWrite(M1_P1_DIR, LOW);
  pinMode(M1_P1_EN, OUTPUT);
  digitalWrite(M1_P1_EN, LOW);

  pinMode(M1_P2_DIR, OUTPUT);
  digitalWrite(M1_P2_DIR, LOW);
  pinMode(M1_P2_EN, OUTPUT);
  digitalWrite(M1_P2_EN, LOW);

  pinMode(M1_P3_DIR, OUTPUT);
  digitalWrite(M1_P3_DIR, LOW);
  pinMode(M1_P3_EN, OUTPUT);
  digitalWrite(M1_P3_EN, LOW);


  pinMode(M2_P1_DIR, OUTPUT);
  digitalWrite(M2_P1_DIR, LOW);
  pinMode(M2_P1_EN, OUTPUT);
  digitalWrite(M2_P1_EN, LOW);

  pinMode(M2_P2_DIR, OUTPUT);
  digitalWrite(M2_P2_DIR, LOW);
  pinMode(M2_P2_EN, OUTPUT);
  digitalWrite(M2_P2_EN, LOW);

  pinMode(M2_P3_DIR, OUTPUT);
  digitalWrite(M2_P3_DIR, LOW);
  pinMode(M2_P3_EN, OUTPUT);
  digitalWrite(M2_P3_EN, LOW);

  pinMode(M1_HALL_1, INPUT);
  pinMode(M1_HALL_2, INPUT);
  pinMode(M1_HALL_3, INPUT);

  pinMode(M2_HALL_1, INPUT);
  pinMode(M2_HALL_2, INPUT);
  pinMode(M2_HALL_3, INPUT);


  // start serial port at 115200 bps:
  Serial.begin(115200);
  establishContact();
}

// the loop routine runs over and over again forever:
void loop() {

  comByte = 0;
  if (Serial.available() > 0){//if valid, read from serial:
    //get incoming byte:
    comByte = Serial.read();
    Serial.print('R');   // send 'R' to initiate next data from computer
  }//if (Serial.available() > 0){

  if(comByte > 0){
    step1 = get_step1();
    step2 = get_step2();

    if(comByte == 't'){
      step_motors(6,6);
      Serial.write("senor1=");
      Serial.print(sensorState1, BIN);
      Serial.write(", senor2=");
      Serial.print(sensorState2, BIN);
      Serial.write("\n");
    }
    if(comByte == 'f'){
      int i;
      stepDelay = STEP_DELAY_MAX;
      for(i=0;i<driveLength;i++){
        step_motors(1,1);
        step_motors(1,1);
      }
      Serial.print(comByte);   // echo 'comByte' to confirm
    }
    if(comByte == 'b'){//Drive backwards
      int i;
      stepDelay = STEP_DELAY_MAX;
      for(i=0;i<driveLength;i++){
        step_motors(-1,-1);
        step_motors(-1,-1);
      }
      Serial.print(comByte);   // echo 'comByte' to confirm
    }
    if(comByte == 'l'){//Turn left
      int i;
      stepDelay = STEP_DELAY_MAX;
      for(i=0;i<driveLength2;i++){
        step_motors(1,-1);
        step_motors(1,-1);
      }
      Serial.print(comByte);   // echo 'comByte' to confirm
    }
    if(comByte == 'r'){//Turn right
      int i;
      stepDelay = STEP_DELAY_MAX;
      for(i=0;i<driveLength2;i++){
        step_motors(-1,1);
        step_motors(-1,1);
      }
      Serial.print(comByte);   // echo 'comByte' to confirm
    }

    if(comByte == 'w'){//front left
      int i;
      stepDelay = STEP_DELAY_MAX;
      for(i=0;i<driveLength/2;i++){
        step_motors(1,1);
        step_motors(1,0);
      }
      Serial.print(comByte);   // echo 'comByte' to confirm
    }

    if(comByte == 'x'){//front right
      int i;
      stepDelay = STEP_DELAY_MAX;
      for(i=0;i<driveLength/2;i++){
        step_motors(1,1);
        step_motors(0,1);
      }
      Serial.print(comByte);   // echo 'comByte' to confirm
    }

    if(comByte == 'y'){//back left
      int i;
      stepDelay = STEP_DELAY_MAX;
      for(i=0;i<driveLength/2;i++){
        step_motors(-1,-1);
        step_motors(-1,0);
      }
      Serial.print(comByte);   // echo 'comByte' to confirm
    }

    if(comByte == 'z'){//back right
      int i;
      stepDelay = STEP_DELAY_MAX;
      for(i=0;i<driveLength/2;i++){
        step_motors(-1,-1);
        step_motors(0,-1);
      }
      Serial.print(comByte);   // echo 'comByte' to confirm
    }

    if(comByte == '+' && setupMode == 0){//Raise motor power
      PWM1 += 5;
      if(PWM1 > PWM1_MAX){
        PWM1 = PWM1_MAX;
      }
      Serial.print(comByte);   // echo 'comByte' to confirm
    }
    if(comByte == '-' && setupMode == 0){//Lower motor power
      PWM1 -= 5;
      if(PWM1 < PWM1_MIN){
        PWM1 = PWM1_MIN;
      }
      Serial.print(comByte);   // echo 'comByte' to confirm
    }

    if(comByte == '+' && setupMode == 1){//Lower Speed
      stepDelay += stepDelayDiff;
      if(stepDelay > STEP_DELAY_MAX){
        stepDelay = STEP_DELAY_MAX;
      }
      Serial.print(comByte);   // echo 'comByte' to confirm
    }
    if(comByte == '-' && setupMode == 1){//Raise Speed
      stepDelay -= stepDelayDiff;
      if(stepDelay < STEP_DELAY_MIN){
        stepDelay = STEP_DELAY_MIN;
      }
      Serial.print(comByte);   // echo 'comByte' to confirm
    }


    if(comByte == 'P'){//Set up PWM with + or -
      setupMode = 0;
      Serial.print(comByte);   // echo 'comByte' to confirm
    }

    if(comByte == 'S'){//Set up speed with + or -
      setupMode = 1;
      Serial.print(comByte);   // echo 'comByte' to confirm
    }


    if(comByte == '1'){//Set driveLength
      driveLength = 15;
      driveLength2 = 6;
      Serial.print(comByte);   // echo 'comByte' to confirm
    }
    if(comByte == '2'){//Set driveLength
      driveLength = 30;
      driveLength2 = 12;
      Serial.print(comByte);   // echo 'comByte' to confirm
    }
    if(comByte == '3'){//Set driveLength
      driveLength = 45;
      driveLength2 = 18;
      Serial.print(comByte);   // echo 'comByte' to confirm
    }
    if(comByte == '4'){//Set driveLength
      driveLength = 60;
      driveLength2 = 24;
      Serial.print(comByte);   // echo 'comByte' to confirm
    }
    if(comByte == '5'){//Set driveLength
      driveLength = 75;
      driveLength2 = 30;
      Serial.print(comByte);   // echo 'comByte' to confirm
    }
    if(comByte == '6'){//Set driveLength
      driveLength = 90;
      driveLength2 = 36;
      Serial.print(comByte);   // echo 'comByte' to confirm
    }
    if(comByte == '7'){//Set driveLength
      driveLength = 105;
      driveLength2 = 42;
      Serial.print(comByte);   // echo 'comByte' to confirm
    }
    if(comByte == '8'){//Set driveLength
      driveLength = 120;
      driveLength2 = 48;
      Serial.print(comByte);   // echo 'comByte' to confirm
    }
    if(comByte == '9'){//Set driveLength
      driveLength = 135;
      driveLength2 = 54;
      Serial.print(comByte);   // echo 'comByte' to confirm
    }
  }


}

