/*
 * RxC1.c:
 * Control program for Rover
 * compile with: gcc RxC1.c -I/usr/local/include -L/usr/local/lib -lwiringPi -lpigpio -lm -o RxC1
 */

//pigpio numbering varries depending on hardware version of Raspberry

#include <wiringPi.h>

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <math.h>
#include <string.h>
#include <termios.h>
#include <fcntl.h>

//#include <sys/ioctl.h>
//#include <fcntl.h>
//#include <sys/stat.h>
//#include <sys/types.h>
#include <unistd.h>
//#include <wiringPiI2C.h>// Activate I2C on Raspberry or else compillation will fail with segmentation fault !!!

#include <pigpio.h>
//#include <time.h>
#include <sys/time.h>


#define BAUDRATE B115200
//#define ARDUINOPORT "/dev/ttyUSB0"
#define ARDUINOPORT "/dev/ttyACM0"

#define SERIAL_DELAY 10000

int fd = 0;
struct termios TermOpt;


//Pin numbers WiringPi
#define LIGHT_1         29//P1-36
#define LIGHT_2         28//P1-38
#define LIGHT_3         27//P1-40
#define LIGHT_4         26//P1-32

#define IR_SENSOR        5//P1-12
#define IR_DIODE         1//P1-18


//Pin numbers pigpio !!!
#define SERVO_CAMERA       8//P1-24


#define SERVO_CAMERA_MIN    59
#define SERVO_CAMERA_MAX   111


#define MAX_LINE_SIZE 1000

//IR communication: Pulse length in ms
#define PULSE_LENGTH 14

long long getmillis(){
  struct timeval tv;
  gettimeofday(&tv, NULL);
  return (tv.tv_sec) * 1000 + (tv.tv_usec)/1000;
}

static void usage(){
  printf("usage:\n");
  printf("Turn on headlights: RxC1 h (0-3)\n");
  printf("Driving backwards: RxC1 b (0-10000)\n");
}

long getIrPulseLenght(){
  struct timeval startTime;
  struct timeval currentTime;
  long long PulseLength = 0;
  long long startMicros = 0;
  long long currentMicros = 0;
  long long diffMicros = 0;
  long long pulseTimeout = PULSE_LENGTH * 5000;

  gettimeofday(&startTime, NULL);
  startMicros = startTime.tv_sec * 1000000 + startTime.tv_usec;
  currentMicros = startMicros;

  while(digitalRead(IR_SENSOR) == 1 && currentMicros - startMicros < pulseTimeout){
    gettimeofday(&currentTime, NULL);
    currentMicros = currentTime.tv_sec * 1000000 + currentTime.tv_usec;
  }
  if(currentMicros - startMicros >= pulseTimeout){
    return -1;
  }

  gettimeofday(&startTime, NULL);
  startMicros = startTime.tv_sec * (int)1000000 + startTime.tv_usec;
  currentMicros = startMicros;
  while(digitalRead(IR_SENSOR) == 0 && currentMicros - startMicros < pulseTimeout){
    gettimeofday(&currentTime, NULL);
    currentMicros = currentTime.tv_sec * 1000000 + currentTime.tv_usec;
  }
  if(currentMicros - startMicros >= pulseTimeout){
    return -1;
  }

  PulseLength = (currentMicros - startMicros)/1000;

  return PulseLength;
}

//+++++++++++++++++++++++ Start readport ++++++++++++++++++++++++++
char  readport(void){
  int n;
  char buff;
  n = read(fd, &buff, 1);
  if(n > 0){
    return buff;
  }
  return 0;
}
//------------------------ End readport ----------------------------------

//+++++++++++++++++++++++ Start sendport ++++++++++++++++++++++++++
void sendport(unsigned char ValueToSend){
  int n;

  n = write(fd, &ValueToSend, 1);

  if (n < 0){
    printf("write() of value failed!\n");
  }
//  else{
//    while(readport() != 'R')usleep(SERIAL_DELAY);
//  }

}
//------------------------ End sendport ----------------------------------

//+++++++++++++++++++++++ Start openport ++++++++++++++++++++++++++
void openport(void){

    fd = open(ARDUINOPORT, O_RDWR | O_NOCTTY | O_NDELAY);
    if (fd == -1)  {
      printf("init_serialport: Unable to open port\n");
    }

    if (tcgetattr(fd, &TermOpt) < 0) {
      printf("init_serialport: Couldn't get term attributes\n");
    }
    speed_t brate = BAUDRATE; // let you override switch below if needed

    cfsetispeed(&TermOpt, brate);
    cfsetospeed(&TermOpt, brate);

    // 8N1
    TermOpt.c_cflag &= ~PARENB;
    TermOpt.c_cflag &= ~CSTOPB;
    TermOpt.c_cflag &= ~CSIZE;
    TermOpt.c_cflag |= CS8;
    // no flow control
    TermOpt.c_cflag &= ~CRTSCTS;

    TermOpt.c_cflag |= CREAD | CLOCAL;  // turn on READ & ignore ctrl lines
    TermOpt.c_iflag &= ~(IXON | IXOFF | IXANY); // turn off s/w flow ctrl

    TermOpt.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG); // make raw
    TermOpt.c_oflag &= ~OPOST; // make raw

    // see: http://unixwiz.net/techtips/termios-vmin-vtime.html
    TermOpt.c_cc[VMIN]  = 0;
    TermOpt.c_cc[VTIME] = 20;

    if( tcsetattr(fd, TCSANOW, &TermOpt) < 0) {
      printf("init_serialport: Couldn't set term attributes\n");
    }

}
//------------------------ End openport ----------------------------------



//######################################################################
//######################################################################
//######################################################################

int main (int argc, char **argv){
  int i;
  char charTemp[1000]="";
  int Verbose = 0;
  struct timeval structStartTime, structEndTime;
  unsigned long startTime, endTime;

  int ServoL = 90;
  int ServoR = 90;
  int ServoCamera = 90;

  FILE *IRID;
  char IrFileName[]="IrModuleInRange.txt";
  int moduleNumber = 0;




  if(argc<3){
    usage();
    return(0);
  }

  if(argc>3){
    if(strcmp(argv[3], "v") == 0){
      Verbose = 1;
    }
    if(argc>4){
      if(atoi(argv[4]) > 1){
        Verbose = atoi(argv[4]);
      }
    }
  }

  if(Verbose == 1){
    printf ("Rx Control program\n") ;
    printf("argc=%d, argv[1]=%s, argv[2]=%s\n", argc, argv[1], argv[2]);
  }

  if (wiringPiSetup () == -1){
    printf("Could not set up wiringPi!");
    exit(1);
  }

  if (gpioInitialise() < 0){
    printf("Could not run gpioInitialise!");
    exit(1);
  }


  pinMode (LIGHT_1, OUTPUT);
  pinMode (LIGHT_2, OUTPUT);
  pinMode (LIGHT_3, OUTPUT);
  pinMode (LIGHT_4, OUTPUT);

  pinMode (IR_SENSOR, INPUT);   //IR receiver
  pullUpDnControl (IR_SENSOR, PUD_OFF);

  //Only GPIO 1 is for PWM hardware
  //Frequency = 19.2MHz / pwmSetClock / pwmSetRange
  pinMode (IR_DIODE, PWM_OUTPUT);   //IR diode
  pwmSetMode (PWM_MODE_MS);
  pwmSetClock (5);
  pwmSetRange (101);



  if(strcmp(argv[1], "c") == 0){//Camera servo
    ServoCamera = atoi(argv[2]);
    if(ServoCamera>SERVO_CAMERA_MIN && ServoCamera<SERVO_CAMERA_MAX){
      printf("Adjusting camara to %d\n", ServoCamera);
      gpioServo(SERVO_CAMERA, (ServoCamera) * 1000 / 90 + 500);
      usleep(1000000);
      gpioServo(SERVO_CAMERA, 0);
    }
    else{
      printf("ServoCamera=%ld out of range!\n", ServoCamera);
    }
  }

  // Commands send to Arduino
  if(strcmp(argv[1], "f") == 0 ||
     strcmp(argv[1], "b") == 0 ||
     strcmp(argv[1], "tl") == 0 ||
     strcmp(argv[1], "tr") == 0 ||
     strcmp(argv[1], "fl") == 0 ||
     strcmp(argv[1], "fr") == 0 ||
     strcmp(argv[1], "bl") == 0 ||
     strcmp(argv[1], "br") == 0
     ){


    // Establish connection to Arduino
    openport();
    printf("\n\nWaiting for 'X' from Arduino (Arduino pluged in?)...\n");
    //Wait for 'X' from Arduino
    while(readport() != 'X');
    sendport('R');
    printf("\n...connection established.\n");

    long driveTime = atoi(argv[2]);
    char driveLength = 'X';
    if(driveTime == 10){
      driveTime = 9;
      driveLength = '9';
    }
    else{
      if(driveTime < 10 && driveTime > 0){
        driveLength = argv[2][0];
      }
    }
    printf("driveLength='%c'\n", driveLength);

  //  exit(0); //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    if(strcmp(argv[1], "f") == 0){//Drive forward
      if(driveLength != 'X'){
  //      printf("Flushing Arduino buffer\n");
  //      while(readport() > 0)printf("Received %c", readport());// Empty buffer from Arduino
        sendport(driveLength);
    //    printf("Waiting for driveLength to confirm\n");
    //    while(readport() != driveLength)printf("driveLength Received %c", readport());
        sendport('f');
        printf("Waiting for command to be finished\n");
        while(readport() != 'f')usleep(SERIAL_DELAY);// Wait until command is processed
      }
    }

    if(strcmp(argv[1], "b") == 0){//Drive backward
      if(driveLength != 'X'){
  //      while(readport() > 0);// Empty buffer from Arduino
        sendport(driveLength);
  //      while(readport() != driveLength);
        sendport('b');
        while(readport() != 'b')usleep(SERIAL_DELAY);// Wait until command is processed
      }
    }

    if(strcmp(argv[1], "tl") == 0){//Turn left
      if(driveLength != 'X'){
  //      while(readport() > 0);// Empty buffer from Arduino
        sendport(driveLength);
  //      while(readport() != driveLength);
        sendport('l');
        while(readport() != 'l')usleep(SERIAL_DELAY);// Wait until command is processed
      }
    }

    if(strcmp(argv[1], "tr") == 0){//Turn right
      if(driveLength != 'X'){
  //      while(readport() > 0);// Empty buffer from Arduino
        sendport(driveLength);
  //      while(readport() != driveLength);
        sendport('r');
        while(readport() != 'r')usleep(SERIAL_DELAY);// Wait until command is processed
      }
    }

    if(strcmp(argv[1], "fl") == 0){//front left
      if(driveLength != 'X'){
  //      while(readport() > 0);// Empty buffer from Arduino
        sendport(driveLength);
  //      while(readport() != driveLength);
        sendport('w');
        while(readport() != 'w')usleep(SERIAL_DELAY);// Wait until command is processed
      }
    }

    if(strcmp(argv[1], "fr") == 0){//front right
      if(driveLength != 'X'){
  //      while(readport() > 0);// Empty buffer from Arduino
        sendport(driveLength);
  //      while(readport() != driveLength);
        sendport('x');
        while(readport() != 'x')usleep(SERIAL_DELAY);// Wait until command is processed
      }
    }

    if(strcmp(argv[1], "bl") == 0){//back left
      if(driveLength != 'X'){
  //      while(readport() > 0);// Empty buffer from Arduino
        sendport(driveLength);
  //      while(readport() != driveLength);
        sendport('y');
        while(readport() != 'y')usleep(SERIAL_DELAY);// Wait until command is processed
      }
    }

    if(strcmp(argv[1], "br") == 0){//back right
      if(driveLength != 'X'){
  //      while(readport() > 0);// Empty buffer from Arduino
        sendport(driveLength);
  //      while(readport() != driveLength);
        sendport('z');
        while(readport() != 'z')usleep(SERIAL_DELAY);// Wait until command is processed
      }
    }
  }// Commands send to Arduino



  if(strcmp(argv[1], "h") == 0){//headlight control
    if(atoi(argv[2]) == 0){
      digitalWrite(LIGHT_1, 0);
      digitalWrite(LIGHT_2, 0);
      digitalWrite(LIGHT_3, 0);
      digitalWrite(LIGHT_4, 0);
    }
    if(atoi(argv[2]) == 1){
      digitalWrite(LIGHT_1, 1);
      digitalWrite(LIGHT_2, 0);
      digitalWrite(LIGHT_3, 0);
      digitalWrite(LIGHT_4, 0);
    }
    if(atoi(argv[2]) == 2){
      digitalWrite(LIGHT_1, 1);
      digitalWrite(LIGHT_2, 1);
      digitalWrite(LIGHT_3, 0);
      digitalWrite(LIGHT_4, 0);
    }
    if(atoi(argv[2]) == 3){
      digitalWrite(LIGHT_1, 1);
      digitalWrite(LIGHT_2, 1);
      digitalWrite(LIGHT_3, 1);
      digitalWrite(LIGHT_4, 0);
    }
    if(atoi(argv[2]) == 4){
      digitalWrite(LIGHT_1, 1);
      digitalWrite(LIGHT_2, 1);
      digitalWrite(LIGHT_3, 1);
      digitalWrite(LIGHT_4, 1);
    }
  }

  if(strcmp(argv[1], "test") == 0){
    pinMode (IR_DIODE, OUTPUT);

    printf ("IR ON...\n");
    digitalWrite(IR_DIODE, 1);
    sleep(20);
    digitalWrite(IR_DIODE, 0);
    printf ("..IR OFF!\n");

  }

  if(strcmp(argv[1], "r") == 0){

    long pulseLength[16];

    //Only GPIO 1 is for PWM hardware
    //Frequency = 19.2MHz / pwmSetClock / pwmSetRange
    pinMode (IR_DIODE, PWM_OUTPUT);   //IR diode
    pwmSetMode (PWM_MODE_MS);
    pwmSetClock (5);
    pwmSetRange (106);


    if((IRID=fopen(IrFileName,"wb"))==NULL){
      printf("Can't open file '%s'!\n", IrFileName);
      return(1);
    }
    else{
      sprintf(charTemp, "Module in range = 000");
      fwrite(&charTemp, 21, 1, IRID);
      fclose(IRID);
    }




//printf ("IR ON...\n");

    //Turn on IR diode for 10ms to request modules for answer with module ID
    pwmWrite (IR_DIODE, 50);
    usleep(PULSE_LENGTH * 10000);
    pwmWrite (IR_DIODE, 0);
//printf ("..IR OFF!\n");


    usleep(PULSE_LENGTH * 500);

    for(int i=0; i<16; i++){
      pulseLength[i] = getIrPulseLenght();
      if(pulseLength[i]<0)break;
    }

    //for(int i=0; i<16; i++){
    //  printf("pulseLength[%d] = %ld\n", i, pulseLength[i]);
    //}

    int id1 = 0;
    int id2 = 0;
    for(int i=0; i<16; i++){
      if(pulseLength[i] < 0)break;
      if(i<8){
        if(pulseLength[i] > PULSE_LENGTH * 1.5){
          id1 += pow(2, i);
        }
      }
      else{
        if(pulseLength[i] > PULSE_LENGTH * 1.5){
          id2 += pow(2, i-8);
        }
      }
    }
    //printf("id1=%d, id2=%d\n", id1, id2);


    //Send number of command pulses
    if(atoi(argv[2]) > 0){
      usleep(PULSE_LENGTH*1000); //Wait until microcontroller is prepared to receive command pulses
      for(i=0; i < atoi(argv[2]); i++){
        pwmWrite (IR_DIODE, 50);
        usleep(PULSE_LENGTH * 1000);
        pwmWrite (IR_DIODE, 0);
        usleep(PULSE_LENGTH * 1000);
        if(Verbose==1){
          printf("Sent pulse number %d\n", i);
        }
      }
    }

    if(Verbose > 0){
      printf ("Module found = %d\n", id1);
    }

    if((IRID=fopen(IrFileName,"wb"))==NULL){
      //printf("Can't open file '%s'!\n", IrFileName);
      return(1);
    }
    else{
      sprintf(charTemp, "Module in range = %03d", id1);
      fwrite(&charTemp, 21, 1, IRID);
      fclose(IRID);
    }

  }//if(strcmp(argv[1], "r") == 0){


  close(fd);// Close USB port to Arduino

  return 0 ;
}













